<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" queryBinding="xslt2">
    <ns uri="http://www.loc.gov/mads/rdf/v1#" prefix="madsrdf"/>
    <ns uri="http://ead3.archivists.org/schema/" prefix="ead"/>
    
    <!-- Extending rules for the use of EAD3 following "FGS Arkivredovisning (Verksamhets-baserad)Förvaltningsgemensam specifikation (FGS) för Arkivredovisning (Verksamhetsbaserad)RAFGS5V0.1"
        2017-10-25 -->
    
    <extends href="ead3.sch"/>

    <pattern id="control">
        <rule context="ead:control">
            <!-- FGS use need to have this set of attributes and elements present -->
            <assert test="exists(@countryencoding)">Control need to have the attribute countryencoding</assert>
            <assert test="exists(@langencoding)">Control need to have the attribute langencoding</assert>
            <assert test="exists(@repositoryencoding)">Control need to have the attribute repositoryencoding</assert>
            <assert test="exists(@dateencoding)">Control need to have the attribute dateencoding</assert>
            <assert test="exists(@scriptencoding)">Control need to have the attribute scriptencoding</assert>
            <assert test="count(ead:conventiondeclaration) >= 2">At least two convention declarations are needed</assert>
            <assert test="ead:localtypedeclaration">One localtype declaration is needed</assert>
            <assert test="ead:languagedeclaration">One language declaration is needed</assert>
        </rule>
    </pattern>
    
    <pattern id="conventiondeclaration">
        <!-- A convention declaration in FGS use needs these elements and attributes -->
        <rule context="ead:conventiondeclaration">
            <assert test="exists(ead:citation)">The convention declaration needs a citation element</assert>
            <assert test="exists(ead:citation/@href)">The convention declaration element citation needs a href attribute</assert>
            <assert test="exists(ead:abbr)">The convention declaration needs an abbrivation</assert>
            <assert test="exists(ead:descriptivenote/ead:p)">The convention declaration needs a descriptive note with a p element</assert>
        </rule>
    </pattern>
    
    <pattern id="languagedeclaration">
        <!-- The language declaration looks like this -->
        <rule context="ead:languagedeclaration">
            <assert test="exists(ead:language)">The language declaration needs a language element</assert>
            <assert test="exists(ead:language/@langcode)">The language element needs an langcode attribute</assert>
            <assert test="exists(ead:script)">The language declaration needs a script element</assert>
            <assert test="exists(ead:script/@scriptcode)">The script element needs a scriptcode attribute</assert>
        </rule>
    </pattern>
    
    <pattern id="localtypedeclaration">
        <!-- The localtype declaration looks like this -->
        <rule context="ead:localtypedeclaration">
            <assert test="exists(ead:citation)">The localtype declaration needs a citation element</assert>
            <assert test="exists(ead:citation/@href)">The localtype declaration element citation needs a href attribute</assert>
            <assert test="exists(ead:abbr)">The localtype declaration needs an abbrivation</assert>
            <assert test="exists(ead:descriptivenote/ead:p)">The localtype declaration needs a descriptive note element</assert>
        </rule>
    </pattern>
    
    <pattern id="origination">
        <!-- When there are an originator it needs to be a creator -->
        <rule context="ead:origination">
            <assert test="@localtype = 'creator'">When there are an origination present one with the attribute localtype with the value creator needs to exist</assert>
            <assert test="exists(node()/@identifier)">An identfier attribute must be present</assert>
        </rule>
    </pattern>
    
    <pattern id="did">
        <!-- All did in dsc needs to have a unitid and a unittitle-->
        <rule context="ead:dsc">
            <assert test="exists(descendant::ead:unittitle)">A unittitle needs to be present in the did</assert>
            <assert test="exists(descendant::ead:unitid)">A unitid needs to be present in the did</assert>
        </rule>
    </pattern>
    
    <pattern id="archdesc">
        <!-- The archdesc needs these elements and attributes -->
        <rule context="ead:archdesc">
            <assert test="exists(ead:did/ead:unitid)">A unitid needs to be present in archdesc</assert>
            <assert test="exists(ead:did/ead:unitid/@countrycode)">The unitid in archdesc need to have an attribute countrycode present</assert>
            <assert test="exists(ead:did/ead:unitid/@repositorycode)">The unitid in archdesc need to have an attribute repositorycode present</assert>
            <assert test="exists(ead:did/ead:physdescset)">A physdescset needs to be present in archdesc</assert>
            <assert test="exists(ead:did/ead:physdescset/@parallel)">The physdescset element needs to have an attribute parallel present</assert>
            <assert test="exists(ead:did/ead:physdescset/@coverage)">The physdescset element needs to have an attribute coverage present</assert>
            <assert test="count(ead:did/ead:physdescset/ead:physdescstructured) >= 2">At least two physdescstructured is needed at this level in the description</assert>
            <assert test="exists(ead:did/ead:physdescset/ead:physdescstructured/@coverage)">The physdescstructured element needs to have an attribute coverage present</assert>
        </rule>
    </pattern>
    
    <pattern id="pnnn">
        <!-- We need at least one p in all these elements -->
        <rule context="ead:accessrestrict">
            <assert test="count(ead:p) >= 1">The accessrestrict needs to contain one or more p</assert>
        </rule>
        <rule context="ead:userestrict">
            <assert test="count(ead:p) >= 1">The userestrict needs to contain one or more p</assert>
        </rule>
        <rule context="ead:appriasal">
            <assert test="count(ead:p) >= 1">The apprisal needs to contain one or more p</assert>
        </rule>
        <rule context="ead:acqinfo">
            <assert test="count(ead:p) >= 1">The acqinfo needs to contain one or more p</assert>
        </rule>
        <rule context="ead:separatedmaterial">
            <assert test="count(ead:p) >= 1">The separetedmaterial needs to contain one or more p</assert>
        </rule>
        <rule context="ead:relatedmaterial">
            <assert test="count(ead:p) >= 1">The relatedmaterial needs to contain one or more p</assert>
        </rule>
        <rule context="ead:scopecontent">
            <assert test="count(ead:p) >= 1">The scopecontent needs to contain one or more p</assert>
        </rule>
        <rule context="ead:arrangement">
            <assert test="count(ead:p) >= 1">The arrangement needs to contain one or more p</assert>
        </rule>
    </pattern>
    
    <pattern id="didnote">
        <!-- We type the didnote with these values -->
        <rule context="ead:didnote">
            <assert test="@localtype = 'securityclass' or @localtype = 'responsible' or @localtype = 'structuralunitdescription'">The element didnote needs to have the attribute localtype set to securityclass or responsible</assert>
        </rule>
    </pattern>
    
    <pattern id="cid">
        <!-- All c needs an id -->
        <rule context="ead:c">
            <assert test="exists(@id)">The c needs an id attribute</assert>
           </rule>
    </pattern>
    
    <pattern id="cotherlevelrecordkeepingenteties">
        <!-- The otherlevel attriubte except when its a record keeping entity of some sort needs to follow the preset list-->
        <rule context="ead:c[not(ancestor-or-self::ead:c[@otherlevel='recordkeepingentity'])]">
            <assert test="@otherlevel = 'classificationstructure' or @otherlevel = 'structuralunit' or @otherlevel = 'recordgroup' or @otherlevel = 'recordtype' or @otherlevel = 'recordkeepingentity'">The otherlevel attribute needs to follow the preset list vcOTHERLEVEL</assert>
        </rule>
    </pattern>
    
    <pattern id="classificationstructure">
        <!-- The classificationstructure requires these elements and attributes -->
        <rule context="ead:c[@otherlevel = 'classificationstructure']">
            <assert test="exists(ead:did/ead:unitid/@label)"> The classification structure needs to have an atrtibute level giving the version of the classification structure</assert>
            <assert test="exists(ead:did/ead:unitdatestructured/ead:daterange)">The classification structure needs to have an unitdatestructured with a daterange describing its its valid dates</assert>
            <assert test="exists(ead:legalstatus/ead:p/ead:date)">The classification structure need to have a date for the decision of the classification structure</assert>
        </rule>
    </pattern>
    
    <pattern id="structuralunitlegalstatus">
        <!-- The structutal unit needs a date for its decision -->
        <rule context="ead:c[@otherlevel = 'structuralunit']/ead:legalstatus">
            <assert test="exists(ead:p/ead:date)">The structutal unit needs to have a date for the change of the decision of the structural unit</assert>
        </rule>
    </pattern>
   
    <pattern id="recordkeepingentity">
        <!-- The grouping element for record keeping enteties looks like this -->
        <rule context="ead:c[@otherlevel = 'recordkeepingentity']">
            <assert test="contains('FE', ead:did/ead:unitid)">The grouping of record keeping entities have the unitid FE</assert>
            <assert test="contains('Grouping of record keeping entities', ead:did/ead:unittitle)">The grouping of record keeping entities have the unittitle Grouping of record keeping entities</assert>
        </rule>
    </pattern>
    
    <pattern id="unitidref">
        <!-- When a relation is  reted these attributes and elements needs to be present -->
        <rule context="ead:unitid[@localtype = 'relations']">
            <assert test="ead:ref">A relation to another unit needs the element ref</assert>
            <assert test="ead:ref/@target">The relation needs an attribute target</assert>
            <assert test="ead:ref[@linkrole = 'internal']">The relation needs to have the attribute linkrole defined as internal using the value internal</assert>
        </rule>
    </pattern>
    
    <pattern id="unitiddatabase">
        <!-- When its a database compilation described these elements and attributes needs to be present -->
        <rule context="ead:unitid[@localtype = 'databasecompilation']">
            <assert test="ead:ref">A relation to the unit containing the database needs the element ref</assert>
            <assert test="ead:ref/@target">The relation needs an attribute target</assert>
            <assert test="ead:ref[@linkrole = 'internal']">The relation needs to have the attribute linkrole defined as internal using the value internal</assert>
            <assert test="ancestor::ead:c/ead:scopecontent/ead:p">The scopecontent element p describing the information in the recordtype is needed when it is made of a database compilation</assert>
        </rule>
    </pattern>
    
</schema>
